package xelfi.awt;

import java.util.Vector;

import xelfi.util.Debug;

/**
 * A general purpose class for maintaing tree date structure.
 * Each tree node can have multiple children.
 * User can insert to the tree after specified node.
 * This structure supports also sequencial traversing like through array.
	Last change:  DF   17 Mar 97   12:24 pm
*/
public class XFTree implements XelfiElementComparer {
   // root node
   XFTreeNode      root;

   /**
    * Constructs a tree with only one - root node
   */
   public XFTree (Object rootData) {
     root = new XFTreeNode(null, rootData);
   }

   /**
    * Constructs a tree with empty root node
   */
   public XFTree () {
     this(null);
   }

   /**
    * Creates and returns new node after specified node.
   */
   public XFTreeNode insertAfter (Object data, XFTreeNode after) {
     XFTreeNode parent = after.getParent();
     return parent==null ? null : parent.insert(data, after);
   }

   /**
    * Creates and returns new node inserted into specified parent.
   */
   public XFTreeNode insertInto (Object data, XFTreeNode parent) {
     return parent.insert(data, null);
   }

   /**
    * Creates and returns new node after node with specified data
   */
   public XFTreeNode insertAfter (Object data, Object after) {
     XFTreeNode node = findNodeWithData(this, after);
     if (node==null) return null;
     XFTreeNode parent = node.getParent();
     return parent==null ? null : parent.insert(data, node);
   }

   /**
    * Creates and returns new node into node with which contains
    * specified data
   */
   public XFTreeNode insertInto (Object data, Object parent) {
     XFTreeNode node = findNodeWithData(this, parent);
     return node==null ? null : node.insert(data, null);
   }

   /**
    * Deletes specified node.
    * Returns false if trying to delete root node
   */
   public boolean remove (XFTreeNode node) {
     return node.getParent()==null ? false
                                   : node.getParent().remove(node);
   }

   /**
    * Deletes node which contains specified data.
    * Returns false if no node with such data found.
   */
   public boolean remove (Object data) {
     XFTreeNode node = findNodeWithData(this, data);
     return (node==null || node.getParent()==null)
             ? false
             : node.getParent().remove(node);
   }

   /**
    * Returns root node
   */
   public XFTreeNode getRoot () {
     return root;
   }

   /**
    * Returns first visible node
   */
   public XFTreeNode getFirstVisible () {
     XFTreeNode node = root.getChild(0);
		 return node;
   }

   /**
    * Finds node with specified data
   */
   public XFTreeNode findNodeWithData (XelfiElementComparer comparer,
                                       Object data) {
		 XFTreeNode root = getRoot();
		 XFTreeNode result;
		 for (int i=0; i<root.getChildCount(); i++) {
		 	 result = root.getChild(i).findNodeWithData(comparer, data);
			 if (result!=null) return result;
		 }
		 return null;
   }

	/**
	 * Moves the source node into destination node
	*/
	public boolean moveItem (XFTreeNode destination,
												   XFTreeNode source, XFTreeNode after) {
		// check if destination and source are different
		if ((destination==null) || (source==null) || 
				(destination==source)) return false;
		// check if destination is not the child of source
		if (source.isChild(destination)) return false;
    remove(source);
		source.setParent(destination);
    destination.insert(source, after);
		return true;
	}

   /**
    * Implementation of XelfiElementComparer interface
    * Needed for searching through the tree
   */
   public boolean compare (Object first, Object second) {
     return first==second;
   }

}



