/** CompilerOptins class
* @author: Filip Dvorak
* @version: 1.3
*/

package xelfi.compiler;

import java.util.*;
import java.io.File;
import java.awt.*;

import xelfi.top.visual.*;
import xelfi.top.*;
import xelfi.util.*;

  /** This class is a panel inserted into xelfi options dialog, and also keeps the settings
  in its static variables. */
public class CompilerOptions extends Panel implements XelfiOptions
{
    /** What to do on compile all action */
  public static final int ALL_EXCEPT = 1;
  public static final int LISTED_ONLY = 2;
  public static final int EDITORS_ONLY = 3;

  private static int onCompileAllAction = ALL_EXCEPT;
  private static boolean recursive = true;

  private static boolean useInternalCompiler = true;
  private static boolean useGparam = true;
  private static String externalCompilerPath = initExternalCompilerPath();

  private static Vector packList = new Vector();  // initialized in register method

  private static String initInterpretPath()
  {
    String tmp = PathScanner.addFileSeparator(System.getProperty("java.home"));
    return tmp + "bin" + File.separator + "java";
  }

  private static String initExternalCompilerPath()
  {
    String tmp = PathScanner.addFileSeparator(System.getProperty("java.home"));
    return tmp + "bin" + File.separator + "javac -g";
  }

    /** Registers new XelfiOptions in the TopLevel */
  public static void register()
  {
    packList.addElement("java");
    packList.addElement("sun");
    TopLevel.addOption(new CompilerOptions(true));
  }

    /** For back compatibility only, this option has moved to RunOptions */
  public static String getMainClass()
  {
    return xelfi.execution.RunOptions.getMainClass();
  }

    /** Get current setting of one option. */
  public static int onCompileAll()
  {
    return onCompileAllAction;
  }

    /** Get current setting of one option. */
  public static boolean packListRecursive()
  {
    return recursive;
  }

    /** Get current setting of one option. */
  public static String getExternalCompilerPath()
  {
    return externalCompilerPath;
  }

    /** Get current setting of one option. */
  public static Enumeration getPackageList()
  {
    return packList.elements();
  }

    /** Get current setting of one option. */
  public static boolean getUseGparam()
  {
    return useGparam;
  }

    /** Get current setting of one option. */
  public static boolean getUseInternalCompiler()
  {
    return useInternalCompiler;
  }

    /** Searches package list (is an item of the options) for a package.
    @param pack name of the package to search for
    @return whether the package is or is not on the list
    */
  public synchronized static boolean packListContains(String pack)
  {
    int size = packList.size(), i;
    for(i = 0; i < size; i++)
      if(pack.equals((String)packList.elementAt(i))) return true;
    return false;
  }

    /** static variable backup, when showing this panel on the screen */
  private Vector tmppackList;

    // components
  private CheckboxGroup g1, g2;
  private Checkbox
    ic, ec,
    ug, // use -g param
    ce,  // compile all editors
    cc, // compile all classpath
    compilelist, // compile package list
    plr; // package list recursive ?

  private TextField
    ecp;  // external compiler path

  private Button epl;  // edit package list

    /** Creates a new class.
    @param empty if true panel will be empty - instance will be ised for registrating in TopLevel only.
      if false then full panel will be created (with all components) - instance will be used to show on the screen.
    */
  protected CompilerOptions(boolean empty)
  {
    if(empty) return;

    RowLayout pl = new RowLayout();
    setLayout(pl);
    RowLayoutProperties pp = new RowLayoutProperties(new Insets(2, 4, 2, 6));
    pp.leftSpaceGrowth = 0.0f;

    pp.topSpaceGrowth = 0.02f;
    pp.bottomSpaceGrowth = 0.01f;
    Label lab1 = new Label("Use compiler:");
    add(lab1);
    pl.add(lab1, pp);

      // internal-external compiler panel
    pp.compInsets.left += 30;
    pp.compInsets.bottom += 25;
    pp.topSpaceGrowth = 0;
    pp.bottomSpaceGrowth = 0.32f;
    pp.compGrowth = 0.04f;
    Panel cp = new Panel();
    add(cp);
    pl.add(cp, pp);
    pp.compInsets.left -= 30;
    pp.compInsets.bottom -= 25;
    pp.compGrowth = 0f;

    pp.topSpaceGrowth = 0;
    pp.bottomSpaceGrowth = 0.04f;
    Label lab2 = new Label("On compile all:");
    add(lab2);
    pl.add(lab2, pp);

    g2 = new CheckboxGroup();
    pp.compInsets.left += 30;
    pp.topSpaceGrowth = 0;
    pp.bottomSpaceGrowth = 0.04f;
    pp.rightSpaceGrowth = 1f;
    Checkbox ch1 = new Checkbox("compile all editor windows", g2, false);
    Checkbox ch2 = new Checkbox("compile all classpath except the package list", g2, false);
    Checkbox ch3 = new Checkbox("compile the package list only", g2, false);
    add(ch1);
    add(ch2);
    add(ch3);
    pl.add(ch1, pp);
    pl.add(ch2, pp);
    pl.add(ch3, pp);

    pp.topSpaceGrowth = 0.01f;
    pp.bottomSpaceGrowth = 0.04f;
    Checkbox ch4 = new Checkbox("Consider package list recursive");
    add(ch4);
    pl.add(ch4, pp);
    pp.compInsets.left -= 30;
    pp.rightSpaceGrowth = 0;

    pp.topSpaceGrowth = 0.0f;
    pp.compGrowth = 0.06f;
    pp.bottomSpaceGrowth = 0.34f;
    pp.leftSpaceGrowth = pp.rightSpaceGrowth = 0.4f;
    pp.compInsets.bottom += 25;
    pp.compInsets.top += 6;
    epl = new Button("Edit package list");
    add(epl);
    pl.add(epl, pp);
    pp.leftSpaceGrowth = 0.02f;
    pp.rightSpaceGrowth = 0;
    pp.compInsets.bottom -= 25;
    pp.compInsets.top -= 6;

    // compiler panel
    ColLayout cpl = new ColLayout();
    ColLayoutProperties cpp = new ColLayoutProperties(new Insets(0, 0, 0, 20));
    cp.setLayout(cpl);
    cpp.rightSpaceGrowth = 0.16f;
    Panel clp = new Panel();
    cp.add(clp);
    cpl.add(clp, cpp);
      // right panel
    cpp.rightSpaceGrowth = 0;
    cpp.compGrowth = 0.84f;
    cpp.compInsets = new Insets(0, 0, 0, 0);
    Panel crp = new Panel();
    cp.add(crp);
    cpl.add(crp, cpp);

      // left panel setup
    RowLayout clpl = new RowLayout();
    clp.setLayout(clpl);
    RowLayoutProperties clpp = new RowLayoutProperties(new Insets(0, 0, 4, 0), 0.5f);
    clpp.rightSpaceGrowth = 1f;
    g1 = new CheckboxGroup();
    Checkbox ch5 = new Checkbox("internal (sun.tools.javac)", g1, false);
    Checkbox ch6 = new Checkbox("external", g1, false);
    clp.add(ch5);
    clp.add(ch6);
    clpl.add(ch5, clpp);
    clpp.compInsets.bottom = 0;
    clpl.add(ch6, clpp);

      // right panel setup
    RowLayout crpl = new RowLayout();
    crp.setLayout(crpl);
    RowLayoutProperties crpp = new RowLayoutProperties(new Insets(0, 0, 4, 0), 0.5f);
    Checkbox ch7 = new Checkbox("use -g parameter");
    crp.add(ch7);
    crpl.add(ch7, crpp);
    crpp.compInsets.bottom = 0;
    TextField tf1 = new TextField(34);
    crp.add(tf1);
    crpl.add(tf1, crpp);

      // internal - external compiler
    ic = ch5;
    ec = ch6;
      // use -g param
    ug = ch7;
      // external compiler path
    ecp = tf1;
      // compile all editors
    ce = ch1;
      // compile all classpath
    cc = ch2;
      // compile package list
    compilelist = ch3;
      // package list recursive ?
    plr = ch4;

    tmppackList = (Vector)packList.clone();

    if(useInternalCompiler) g1.setCurrent(ic);
    else g1.setCurrent(ec);

    ug.setState(useGparam);

    ecp.setText(externalCompilerPath);

    if(onCompileAllAction == ALL_EXCEPT) g2.setCurrent(cc);
    else if(onCompileAllAction == EDITORS_ONLY) g2.setCurrent(ce);
    else g2.setCurrent(compilelist);

    plr.setState(recursive);
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == epl)  // edit package list
    {
      new PackageListDialog(this).show();
      return true;
    }
    return super.action(e, arg);
  }

  void setTmpList(Vector newVector)
  {
    tmppackList = newVector;
  }

  Vector getTmpList()
  {
    return (Vector)tmppackList.clone();
  }

    /** XelfiOptions implementation */
  public void loadProperties(EnhProperties src)
  {
    useInternalCompiler = src.getBooleanProperty("UseInternalCompiler", useInternalCompiler);

    String tmp;
    tmp = (String)src.get("ExternalCompilerPath");
    externalCompilerPath = tmp == null ? externalCompilerPath : tmp;

    useGparam = src.getBooleanProperty("Use_g_param", useGparam);
    recursive = src.getBooleanProperty("CompilePackageListRecursive", recursive);

    onCompileAllAction = src.getIntProperty("OnCompileAllAction", onCompileAllAction);

    int size = src.getIntProperty("PackageListSize", - 1);
    Vector newpacklist = null;
    if(size != -1)
    {
      newpacklist = new Vector(size);
      for(int i = 0; i < size; i++)
        if((tmp = (String)src.get("PackageListItem" + i)) != null) newpacklist.addElement(tmp);
      setStaticPackageVector(newpacklist);
    }
  }

    /** XelfiOptions implementation */
  public void storeProperties(EnhProperties dest)
  {
    dest.put("ExternalCompilerPath", externalCompilerPath);

    dest.setBooleanProperty("UseInternalCompiler", useInternalCompiler);
    dest.setBooleanProperty("Use_g_param", useGparam);
    dest.setBooleanProperty("CompilePackageListRecursive", recursive);

    dest.setIntProperty("OnCompileAllAction", onCompileAllAction);

    dest.setIntProperty("PackageListSize", packList.size());
    for(int i = 0; i < packList.size(); i++)
      dest.put("PackageListItem" + i, (String)packList.elementAt(i));
  }

    /** XelfiOptions implementation */
  public Panel getPanel()
  {
    return new CompilerOptions(false);
  }

    /** XelfiOptions implementation */
  public String getTabName()
  {
    return "Compiler";
  }

    /** XelfiOptions implementation */
  public void onOK(Panel p)
  {
    ((CompilerOptions)p).saveValues();
  }

  private void saveValues()
  {
    setStaticPackageVector(tmppackList);

    useInternalCompiler = g1.getCurrent() == ic;

    useGparam = ug.getState();

    externalCompilerPath = ecp.getText();

    if(g2.getCurrent() == cc) onCompileAllAction = ALL_EXCEPT;
    else if(g2.getCurrent() == ce) onCompileAllAction = EDITORS_ONLY;
    else onCompileAllAction = LISTED_ONLY;

    recursive = plr.getState();
  }

  private static synchronized void setStaticPackageVector(Vector newVector)
  {
    packList = newVector;
  }
}



