/**
* Xelfi
*/

package xelfi.design.components.dialogs;

import java.awt.*;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.top.TopLevel;

/**
 * StringArraySelectThread class is here only to get working modal XFStringArrayDialog 
 * with JDK 1.02 under Win32 (Sunsoft claimed that this bug is fixed in JDK 1.02
 * but not under Win32 - the show() method of the Dialog class is still not blocking)
 *
 * @version 	1.2, 16 Dec 1996
 * @author 	Petr Hamernik
 */

public class StringArraySelectThread extends Thread {
	private Frame parent;
	private XelfiDesignComponent xdc;
	private int propNum;
	private Container compParent;
	
	/**
	 * Constructor to set parameters for the thread and for the array string dialog
	 * @param	name			the name of the thread
	 * @param	aParent			parent of the dialog
	 * @param	aXdc			Xelfi component whose StringArray propety to set
	 * @param	aPropNum		index of the StringArray property
	 * @param	aCompParent	parent of the component which validate() method is called 
	 */
	public StringArraySelectThread(String name, Frame aParent, XelfiDesignComponent aXdc, 
										int aPropNum) {
		super(name);		
		parent = aParent;
		xdc = aXdc;
		propNum = aPropNum;
	}

	public void run() {
		Object of = null;
		try {
			of = xdc.getProperty(propNum);
			} catch (XFPropertyIndexOutOfBoundsException e) {
		}
		if (of instanceof String[]) {
			XFStringArrayDialog fsd = new XFStringArrayDialog(parent, (String[]) of);
			fsd.show();
			// for the dialog to run at reasonable speed
			// we must set to this loop minimal priority
			setPriority(MIN_PRIORITY);
			do { } while (fsd.finished != true);
			
			if (fsd.result == XFStringArrayDialog.RES_OK) {
	
				// set the StringArray property and catch exceptions
				try {
					int ret = xdc.setProperty(propNum, fsd.getStrings());
					if ((ret & XFDesignConstants.RSP_OTHER_PROP) != 0)
						TopLevel.getInspector().propertiesChanged();
					} catch (XFPropertyBadValueTypeException e) {	
					} catch (XFPropertyIndexOutOfBoundsException e) {
					}
				}
			}
		else System.out.println("Property number: "+ propNum + " of component: "+ xdc +" is not of type StringArray");
		}
}