/**
* Xelfi
*/

// TO IMPROVE:
//		- default OK button (if possible)
//		- update size when leaving size box

package xelfi.design.components.dialogs;

import java.awt.*;

/**
 * XFFontDialog is a dialog for setting font properties
 * These are: font name, font style (plain/bold/italic), font size
 *
 * @version 	1.01, 06 Oct 1996
 * @author 	Ian Formanek
 */

public class XFFontDialog extends Dialog {
	public static final int RES_OK = 1;
	public static final int RES_CANCEL = 2;

	public boolean finished = false;
	
	public int result = 0;
	public Font font = null;
	public String name;
	public int size = 12;
	public int style = Font.PLAIN;
	public boolean isPlain = true;
	public boolean isBold = false;
	public boolean isItalic = false;

	private Button OKBut, CancelBut;
	private Choice nameBox;
	private FontPanel fontTest;
	private TextField sizeField;
	private Checkbox plainCh, boldCh, italicCh;

	public XFFontDialog(Frame parent) {
		this(parent, null);
		}
	public XFFontDialog(Frame parent, Font aFont) {
		super(parent, "Font Selection", true);
		setResizable(false);

		// retrieve font names from system
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		String fontNames[] = toolkit.getFontList();

		if (aFont == null) {
			name = new String(fontNames[0]);
			font = new Font(name,style,size);
			}
		else {
			font = aFont;
			name = font.getName();
			style = font.getStyle();
			size = font.getSize();
			isPlain = font.isPlain();
			isItalic = font.isItalic();
			isBold = font.isBold();
			}
			
		// create and set a layout
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		setLayout(gbl);

		// create all components
		nameBox = new Choice();
		for (int i=0; i<fontNames.length; i++)
			nameBox.addItem(fontNames[i]);
		sizeField = new TextField(Integer.toString(size),2);
		fontTest = new FontPanel(font);
		OKBut = new Button("OK");
		CancelBut = new Button("Cancel");
		plainCh = new Checkbox("Plain");
		boldCh = new Checkbox("Bold");
		italicCh = new Checkbox("Italic");




		// setup constraints and add all the components
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(2,2,2,2);
		c.gridwidth = 3;
		c.anchor = GridBagConstraints.NORTH;
		c.weighty = 0;
		c.weightx = 1;
		gbl.setConstraints(nameBox,c);
		add(nameBox);

		c.weightx = 0;
		c.gridwidth = 2;
		gbl.setConstraints(sizeField,c);
		add(sizeField);

		Panel panel1 = new Panel();
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel1.add(plainCh);
		panel1.add(boldCh);
		panel1.add(italicCh);

		c.fill = GridBagConstraints.BOTH;
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.gridy = 1;

		gbl.setConstraints(panel1,c);
		add(panel1);

		c.weighty = 1;
		c.gridy = 2;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(fontTest,c);
		add(fontTest);

		Panel panel = new Panel();
		panel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		panel.add(OKBut);
		panel.add(CancelBut);
		c.gridy = GridBagConstraints.RELATIVE;
		c.weightx = 0.0;
		c.weighty = 0.0;
     c.fill = GridBagConstraints.HORIZONTAL;
     c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(panel, c);
		add(panel);


		nameBox.select(name);
		updateChecks(); // sets plain/bold/italic boxes according to style
		pack();
		}

	public Dimension preferredSize() { return new Dimension(350,220); } 

	public Dimension minimalSize() { return new Dimension(200,100); } 

    public synchronized void show() {
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		Dimension dialogSize = size();
		move((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
		super.show();
    }

	
	public boolean action(Event event, Object arg) {
		if (event.target == OKBut) {
			result = RES_OK;
			finished = true;
			hide();
			dispose();
			}
		else if (event.target == CancelBut) {
			result = RES_CANCEL;
			finished = true;
			hide();
			dispose();
			}
		else if (event.target == nameBox) {
			name = nameBox.getItem(nameBox.getSelectedIndex());
			updateFont();
		}
		else if (event.target == sizeField) {
			Integer tempSize = null;
			try {
				tempSize = new Integer(sizeField.getText());
			} catch (NumberFormatException e) {
			}

			if (tempSize != null) {
				size = tempSize.intValue();
				updateFont();
			}
		}
		else if (event.target == plainCh) { // can be only checked
			isPlain = true;
			isBold = isItalic = false;
			updateChecks();
			updateFont();
		}
		else if (event.target instanceof Checkbox) {
			isBold = boldCh.getState();
			isItalic = italicCh.getState();
			if ((isBold == false) && (isItalic == false)) isPlain = true;
			else isPlain = false;
			updateChecks();
			updateFont();
		}

		return true;
		}

	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY)
			hide();
		return super.handleEvent(event);
		}

	private void updateFont() {
		font = new Font(name,style,size);
		fontTest.setFont(font);
	}
	private void updateChecks() {
		if (isPlain == true) {
			plainCh.setState(true);
			plainCh.disable();
			italicCh.setState(false);
			boldCh.setState(false);
			style = Font.PLAIN;
			}
		else {
			plainCh.setState(false);
			plainCh.enable();
			style = 0;
			if (isItalic == true) { italicCh.setState(true); style += Font.ITALIC; }
			if (isBold == true) { boldCh.setState(true); style += Font.BOLD; }
			}
		}

}

class FontPanel extends Panel {
	Label label;
	FontPanel(Font aFont) {
		super();
		setLayout(new BorderLayout());
		label = new Label("Font test", Label.CENTER);
		label.setFont(aFont);
		add("Center",label);
		}

	public void setFont(Font aFont) {
		label.setFont(aFont);
		label.invalidate();
		validate();
		}

	public Insets insets() { return new Insets(3,3,3,4); }

	public void paint(Graphics g) {
		Dimension d = size();
		g.setColor(getBackground());
		g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
		g.draw3DRect(2, 2, d.width - 5, d.height - 5, false);
		}
}