/**
* Xelfi
*/

package xelfi.design.components.palette;

/**
 * @version 	1.6 07 Dec 1996
 * @author		Petr Hamernik
 */

import java.awt.*;
import java.util.*;
import xelfi.awt.*;
import xelfi.design.components.*;
import xelfi.design.util.*;
import xelfi.util.XelfiPath;
import xelfi.util.EnhProperties;
import xelfi.top.TopLevel;
import xelfi.top.MainMenu;

public class XFComponentPalette extends Panel implements XelfiComponentPalette {
	private final String magicStr = "CMPL";
    private PageControl pages;
    private ImageCheckButton arrowButton;
    private int mode;
    protected Color marginColor = null;
    protected static final int defaultBorderWidth = 1;
    protected static final Color defaultBorderColor = new Color(196, 196, 196);
    protected static final int indent = 4;
    protected Vector compInfo;
	protected Vector menuItems;
    protected String selComp = null;
    private Dimension dimLast = null;

    public XFComponentPalette() {
        setLayout(null);
		
		Image ic = Toolkit.getDefaultToolkit().getImage(getPaletteIcon("arrow.gif"));
        arrowButton = new ImageCheckButton(ic);
        add(arrowButton);
        validate();

        pages = new PageControl();
        add(pages);
        validate();

        compInfo = new Vector();
		menuItems = new Vector();
        dimLast = new Dimension(-1,-1);
        setDefaults();
    }

    public int getPagesCount() {
        return pages.getTabCount();
    }

	public int getPageIndex() {
		return pages.getTabIndex();
	}
	
    public String getPageName(int index) throws ArrayIndexOutOfBoundsException {
        return pages.getTabName(index);
    }

    public void addPageAt(String name, int position)  throws ArrayIndexOutOfBoundsException {
        Panel p = new Panel();
        int compSize = arrowButton.preferredSize().width;
        p.setLayout(new XFPaletteLayout(4, compSize , compSize));
        pages.addPageAt(name, p, position);
        Vector v = new Vector();
        compInfo.insertElementAt((Object) v, position);
        setUpSizes();
        repaint();
    }

    public void addPage(String name) {
        addPageAt(name, getPagesCount());
    }

    public void paint(Graphics g) {
  		Dimension s = size();
  		if (s != dimLast)
  		    setUpSizes();
        g.setColor(marginColor);
  		g.draw3DRect(0,0,s.width - 1,s.height - 1, false);
  		g.draw3DRect(1,1,s.width - 3,s.height - 3, true);
    }

    private void setDefaults() {
        marginColor = defaultBorderColor;
        pages.setBorderWidth(defaultBorderWidth);
        pages.setBorderColor(defaultBorderColor);
        arrowButton.setBorderWidth(defaultBorderWidth);
        arrowButton.setUpMouseMode(false);
        arrowButton.setState(arrowButton.IMG_BUTTON_DOWN);
		((CheckboxMenuItem) TopLevel.getMenu().getItem(MainMenu.miDesignSelectionMode)).setState(true);
        arrowButton.setBorderColor(marginColor);
        mode = XFDesignConstants.XM_SELECTIONMODE;
    }

    public int getComponentsCount(int page) throws ArrayIndexOutOfBoundsException {
        return ((Vector) compInfo.elementAt(page)).size();
    }

	private String getPaletteIcon(String fileName) {
		String sep = new String(System.getProperty("file.separator"));
		StringBuffer s = new StringBuffer(XelfiPath.get() + sep + 
			"design" + sep + "components" + sep + "palette" + sep + fileName);
		return new String(s);
	}
	
    public void addComponent(int page, String component) throws ArrayIndexOutOfBoundsException {
        addComponentAt(page, getComponentsCount(page), component);
    }

    public void addComponentAt(int page, int position, String component)
        throws ArrayIndexOutOfBoundsException {
        Panel p = pages.getPageAt(page);

		Image icon = null;
		Class xx;
		XelfiDesignComponent xvc = null;
		try {
			xx = Class.forName(component);
			xvc = (XelfiDesignComponent) xx.newInstance();
			icon = xvc.getIcon();
			if (icon == null)
				icon = Toolkit.getDefaultToolkit().getImage(getPaletteIcon("question.gif"));
			PaletteButton b = new PaletteButton(icon, xvc.getClassName());
	        p.add(b);
	        b.validate();
	        b.resize(b.preferredSize().width, b.preferredSize().height);
	        b.setBorderWidth(pages.getBorderWidth());
	        b.setBorderColor(marginColor);
	        Vector v = (Vector) compInfo.elementAt(page);
	        v.insertElementAt((Object) component, position);
		} catch (IllegalAccessException e) {
			System.err.println("Illegal Access Exception");
		} catch (InstantiationException e) {
			System.err.println("Instantiation Exception");
		} catch (ClassNotFoundException e) {
			System.err.println("ClassNotFound Exception:"+component);
		} catch (ClassCastException e) {
			System.err.println("Class Cast Exception:"+component);
		}
    }

    private void setUpSizes() {
        Dimension d = super.size();
        dimLast = d;
        pages.setDirection(false);
        int tmp = arrowButton.preferredSize().width;
        arrowButton.reshape(indent, indent + pages.getBorderWidth(),
            tmp, tmp);
        pages.reshape(2 * indent + tmp, indent, d.width - (3 * indent + tmp + 1), d.height - 2 *  indent);
    }

    public void setBorderColor(Color col) {
		pages.setBorderColor(col);
        marginColor = col;
        arrowButton.setBorderColor(col);
		repaint();
    }

    public Color getBorderColor(Color col) {
        return marginColor;
    }

	public void setBackground(Color c) {
		super.setBackground(c);
		pages.setBackground(c);
		repaint();
	}
	
	public void setFont(Font  f) {
		super.setFont(f);
		pages.setFont(f);
		repaint();
	}
	
	public void setForeground(Color  c) {
		super.setForeground(c);
		pages.setForeground(c);
		repaint();
	}


    public synchronized Dimension minimumSize() {
        return countSize(true);
    }

	public synchronized Dimension preferredSize() {
        return countSize(false);
    }

    private Dimension countSize(boolean min_pref) {
        Dimension d = (min_pref ? pages.minimumSize() : pages.preferredSize());
        Dimension e = (min_pref ? arrowButton.minimumSize() : arrowButton.preferredSize());
        d.width += 3 * indent + e.width;
        d.height = d.height + e.height + 2 * indent;
        return d;
    }

	public void paletteDefault() {
		removeAllPages();
		Vector def = XFComponentPaletteDefault.get();
		for (int i = 0; i < def.size(); i++) {
			Vector v = (Vector) def.elementAt(i);
			addPage((String) v.elementAt(0));
			for (int j = 1; j < v.size(); j++)
				addComponent(i, (String) v.elementAt(j));
		}
	
		setAddMenu();
		invalidate();
		((Component)this).getParent().validate();
	}

	public void setAddMenu() {
		Menu m = (Menu) TopLevel.getMenu().getItem(MainMenu.addMenu);
		if (m == null)
			return;
		for (; m.countItems() > 0;)
			m.remove(0);
		menuItems.removeAllElements();
		
		for (int i = 0; i < compInfo.size(); i++) {
			Menu tmp = new Menu(getPageName(i));
			Vector v = (Vector) compInfo.elementAt(i);
			Vector vtmp = new Vector();
			for (int j = 0; j < v.size(); j++) {
				Class xx;
				XelfiDesignComponent xvc = null;
				try {
					xx = Class.forName((String) v.elementAt(j));
					xvc = (XelfiDesignComponent) xx.newInstance();
					CheckboxMenuItem mi = new CheckboxMenuItem(xvc.getClassName());
					vtmp.addElement(mi);
					tmp.add(mi);
				} catch (IllegalAccessException e) {
					System.err.println("Illegal Access Exception");
				} catch (InstantiationException e) {
					System.err.println("Instantiation Exception");
				} catch (ClassNotFoundException e) {
					System.err.println("ClassNotFound Exception:"+((String) v.elementAt(j)));
				} catch (ClassCastException e) {
					System.err.println("Class Cast Exception:"+((String) v.elementAt(j)));
				}
			}
			if (vtmp.size() == 0) {
				MenuItem mi_null = new MenuItem("(none)");
				mi_null.disable();
				tmp.add(mi_null);
			}
			m.add(tmp);
			menuItems.addElement(vtmp);
		}
		if (compInfo.size() == 0) {
			MenuItem mi_null = new MenuItem("(none)");
			mi_null.disable();
			m.add(mi_null);
		}
	}

//======================================================================

	public void removePageAt(int Page) throws ArrayIndexOutOfBoundsException {
		if (Page == pages.getTabIndex())
			setSelectionMode();
		pages.removePageAt(Page);
	}
	
	public void removeAllPages() {
		pages.removeAllPages();
	    compInfo.removeAllElements();
    	selComp = null;
		setSelectionMode();
	}

	public int getPageIndex(String Name) throws ArrayIndexOutOfBoundsException {
		for (int i = 0; i < pages.getTabCount(); i++) {
			if (pages.getTabName(i).compareTo(Name) == 0)
				return i;
		}
		return -1;
	}
	
	public String getComponentName(int Page, int Position) throws ArrayIndexOutOfBoundsException {
		return (String) ((Vector) compInfo.elementAt(Page)).elementAt(Position);
	}

//======================================================================

    public int getMode() {
        return mode;
    }

    public String getAddComponent() {
        return selComp;
    }

    public void setSelectionMode() {
        if (mode == XFDesignConstants.XM_ADDMODE) {
            setAllButtonsUp(null);
            arrowButton.setState(arrowButton.IMG_BUTTON_DOWN);
			((CheckboxMenuItem) TopLevel.getMenu().getItem(MainMenu.miDesignSelectionMode)).setState(true);
            mode = XFDesignConstants.XM_SELECTIONMODE;
            selComp = null;
        }
    }

	public boolean setAddMode(String comp) {
		for (int i = 0; i < compInfo.size(); i++) {
			Vector v = (Vector) compInfo.elementAt(i);
			for (int j = 0; j < v.size(); j++) {
				if (((String) v.elementAt(j)).compareTo(comp) == 0) {
					mode = XFDesignConstants.XM_ADDMODE;
					pages.setTabIndex(i);
					Panel p = pages.getPageAt(i);
					setAllButtonsUp(null);
					((PaletteButton) p.getComponent(j)).setState(arrowButton.IMG_BUTTON_DOWN);
					try {
						((CheckboxMenuItem) ((Vector) menuItems.elementAt(i)).elementAt(j)).setState(true);
					}
					catch (ArrayIndexOutOfBoundsException e) {
						System.out.println("XFComponentPalette:"+e);
					}
					arrowButton.setState(arrowButton.IMG_BUTTON_UP);
					((CheckboxMenuItem) TopLevel.getMenu().getItem(MainMenu.miDesignSelectionMode)).setState(false);
                    upDateSelComp(j);
					return true;
				}
			}
		}
		return false;
	}

//======================================================================

	public boolean loadPalette(EnhProperties prop) {
		int pag = prop.getIntProperty(magicStr + "PAGES", -1);
		if (pag == -1)
			return false;
		removeAllPages();
		for (int i = 0; i < pag; i++) {
			String pagename = prop.getProperty(magicStr+"PAGENAME"+i, null);
			if (pagename == null)
				continue;
			addPage(pagename);
			int comps = prop.getIntProperty(magicStr + "COMPS"+i, 0);
			for (int j = 0; j < comps; j++) {
				String compname = prop.getProperty(magicStr+"COMP"+i+"-"+j, null);
				if (compname == null)
					continue;
				addComponent(i, compname);
			}
		}
		setUpSizes();
		show();
		invalidate();
		((Component)this).getParent().validate();
		setAddMenu();
		return true;
	}

	public void storePalette(EnhProperties prop) {
		int pag = getPagesCount();
		prop.setIntProperty(magicStr + "PAGES", pag);
		for (int i = 0; i < pag; i++) {
			prop.put(magicStr+"PAGENAME"+i, getPageName(i));
	        Vector v = (Vector) compInfo.elementAt(i);
			prop.setIntProperty(magicStr + "COMPS"+i, v.size());
			for (int j = 0; j < v.size(); j++) {
				prop.put(magicStr+"COMP"+i+"-"+j,(String) v.elementAt(j));
			}
		}
    }

//======================================================================

    protected void setAllButtonsUp(PaletteButton but) {
        if (pages.getTabCount() > 0) {
            Panel p = pages.getPageAt(pages.getTabIndex());
            int nComps = p.countComponents();
  			int checkOn = -1;
		    for (int i = 0; i < nComps; i++) {
                PaletteButton b = (PaletteButton) p.getComponent(i);
                if (but != b) {
                    b.setState(ImageCheckButton.IMG_BUTTON_UP);
				}
                else {
                    upDateSelComp(i);
					checkOn = i;
				}
            }
			for (int k = 0; k < menuItems.size(); k++) {
				Vector v = (Vector) menuItems.elementAt(k);
				for (int i = 0; i < v.size(); i++) {
					((CheckboxMenuItem) v.elementAt(i)).setState(false);
					if ((i == checkOn) && (k == pages.getTabIndex()))
						((CheckboxMenuItem) v.elementAt(i)).setState(true);
				}
			}
        }
    }

    protected void upDateSelComp(int index) {
        Vector v = (Vector) compInfo.elementAt(pages.getTabIndex());
        selComp = (String) v.elementAt(index);
    }

    public boolean handleEvent(Event event) {
        switch (event.id) {
            case PageControl.TAB_CHANGE:
                    if (mode == XFDesignConstants.XM_ADDMODE)
                        setSelectionMode();
                    else
                        setAllButtonsUp(null);
                    return true;
            case ImageCheckButton.IMG_BUTTON_DOWN:
                    if (arrowButton == (ImageCheckButton) event.arg) {
                        setSelectionMode();
                    }
                  else {
                        setAllButtonsUp((PaletteButton) event.arg);
                        arrowButton.setState(ImageCheckButton.IMG_BUTTON_UP);
						((CheckboxMenuItem) TopLevel.getMenu().getItem(MainMenu.miDesignSelectionMode)).setState(false);
                        mode = XFDesignConstants.XM_ADDMODE;
                    }
                    return true;
            case ImageCheckButton.IMG_BUTTON_UP:
                    setSelectionMode();
                    return true;
        }
	    if (event.target instanceof CheckboxMenuItem) {
			for (int i = 0; i < menuItems.size(); i++) {
				Vector v = (Vector) menuItems.elementAt(i);
				for (int j = 0; j < v.size(); j++) {
			    	if (event.target == v.elementAt(j)) {
						try {
							Vector n = (Vector) compInfo.elementAt(i);
					        String s = (String) n.elementAt(j);
							setAddMode(s);
							return true;
						}
						catch (ArrayIndexOutOfBoundsException e) {
							setSelectionMode();
						}
					}
				}
			}
		}
        return super.handleEvent(event);
    }
}
