/**
* Xelfi
*/

package xelfi.design.components.property;

import java.io.*;
import java.awt.Color;
import java.awt.Frame;
import xelfi.design.components.XelfiDesignComponent;
import xelfi.design.components.dialogs.ColorSelectThread;
import xelfi.util.Debug;

/**
 *
 * @version 	1.40, 08 Feb 1997 (from version 1.30 persistence is added)
 * @author 	Ian Formanek, Petr Hamernik
 */
public class XColorPropertyType {
	/**
	 * This method invokes the specialInput Dialog.
	 * To perform this under JDK 1.02, a new thread must be started to do this.
	 * This method returns immediately allowing the AWT thread to run (if called from within
	 * the action method or so...).
	 * The new thread starts the dialog and then waits in a loop for some variable to be set.
	 * Then if user pressed the OK Button method setProperty of the component is called.
	 * @param	parent	parent of the dialog
	 * @param	xdc		Component which property to set
	 * @param	propNum	Index of the property to set
	 */
	public static void invokeSpecialInput(Frame parent, XelfiDesignComponent xdc, int propNum) {
		new ColorSelectThread("ColorSelectThread", parent, xdc, propNum).start(); 
	}

	/**
	 * This method returns name of the color if the color is one of the dafault colors
	 * defined in class Color or null if not.
	 * @param	col		the color which name we are interested in
	 * @return			name of the default color (red, green, ...), null if the
	 *				color is not default
	 */
	public static String getDefaultColorName(Color col) {
		if (col.equals(Color.black)) return "black";
		if (col.equals(Color.blue)) return "blue";
		if (col.equals(Color.cyan)) return "cyan";
		if (col.equals(Color.darkGray)) return "darkGray";
		if (col.equals(Color.gray)) return "gray";
		if (col.equals(Color.green)) return "green";
		if (col.equals(Color.lightGray)) return "lightGray";
		if (col.equals(Color.magenta)) return "magenta";
		if (col.equals(Color.orange)) return "orange";
		if (col.equals(Color.pink)) return "pink";
		if (col.equals(Color.red)) return "red";
		if (col.equals(Color.white)) return "white";
		if (col.equals(Color.yellow)) return "yellow";
		return null;
	}

	public static String viewString(Object property) { 
		String colName = getDefaultColorName((Color) property);
		if (colName != null) return colName;
		return "[" + ((Color)property).getRed() + 
		       ", " + ((Color)property).getGreen() + 
		       ", " + ((Color)property).getBlue() + "]";
	}
	
	public static void saveToStream(Object property, DataOutput stream) throws 
		IOException {
		stream.writeInt(((Color)property).getRGB());
		Debug.debug(Debug.PROPIO_SAVE, "XColorProperty saved: "+viewString(property));
	}

	public static Color loadFromStream(DataInput stream) throws 
		IOException {
		Color color = new Color(stream.readInt());
		Debug.debug(Debug.PROPIO_LOAD, "XColorProperty loaded: "+viewString(color));
		return color;
	}
}
