/**
* Xelfi
*/

package xelfi.design.forms.layouts;

import java.io.*;
import java.awt.GridLayout;
import java.awt.Container;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Point;
import xelfi.design.components.*;
import xelfi.design.components.property.XIntegerPropertyType;
import xelfi.design.util.XFDesignConstants;
import xelfi.util.Debug;

/* 
 * Class XGridLayout 
 *
 * @version 	1.04, 15 Mar 1997
 * @author 	Ian Formanek
 */

public class XGridLayout extends GridLayout implements XelfiDesignLayout {
	private int propertyCount=4;
	private int _hgap, _vgap, _rows, _cols;

	private XelfiVisualContainer parent = null;

	public XGridLayout() {
		super(2,2);
		_hgap = 0; _vgap = 0;
 		_rows = 2; _cols = 2;
	}

	public XGridLayout(int rows, int cols) {
		super(rows, cols);
		_hgap = 0; _vgap = 0;
 		_rows = rows; _cols = cols;
    }

	public XGridLayout(int rows, int cols, int hgap, int vgap) {
		super(rows, cols, hgap, vgap);
		_hgap = hgap; _vgap = vgap;
 		_rows = rows; _cols = cols;
	}
	
	public void setParent(XelfiVisualContainer aParent) {
		parent = aParent;
	}

	public int getPropertyCount() { return propertyCount;}

	public String[] getPropertyNames() { 
		String list[] = new String[propertyCount];
		list[0] = new String("Rows");
		list[1] = new String("Colums");
		list[2] = new String("Horizontal Gap");
		list[3] = new String("Vertical Gap");
		return list; 
	}

	public void addComponent(XelfiVisualComponent comp) {
	}
	
	public void renameComponent(XelfiVisualComponent comp, String newName) throws 
		XFNameNotFoundException {
	}
	
	public void removeComponent(XelfiVisualComponent comp) throws 
		XFNameNotFoundException {
	}

	
	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0:
			case 1:
			case 2:
			case 3: return XFDesignConstants.PVT_INPUT_INT;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object getProperty(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
		switch (index) {
			case 0:	return new Integer(_rows);
			case 1:	return new Integer(_cols);
			case 2:	return new Integer(_hgap);
			case 3:	return new Integer(_vgap);
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
		
	public String getPropertyString(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
			return XIntegerPropertyType.viewString(getProperty(comp, index));
	}
		
	private void makeNewLayout(int rows, int cols, int hgap, int vgap) {
		if (parent != null) {
			XGridLayout newG = new XGridLayout(rows, cols, hgap, vgap);
			parent.setDesignLayout(newG);
		}
	}
	
	public int setProperty(XelfiVisualComponent comp, int index, Object value) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException,
		XFNameNotFoundException {
		int ret = XFDesignConstants.RSP_OK;
		switch (index) {
			case 0: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
						int newRows = ((Integer)value).intValue();
						if (newRows <0) return XFDesignConstants.RSP_DIFF_VALUE;
						if (newRows != _rows)
							makeNewLayout(newRows, _cols, _hgap, _vgap);
						break;
			case 1: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
						int newCols = ((Integer)value).intValue();
						if (newCols <0) return XFDesignConstants.RSP_DIFF_VALUE;
						if (newCols != _cols)
							makeNewLayout(_rows, newCols, _hgap, _vgap);
						break;
			case 2: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
						int newHGap = ((Integer)value).intValue();
						if (newHGap <0) return XFDesignConstants.RSP_DIFF_VALUE;
						if (newHGap != _hgap)
							makeNewLayout(_rows, _cols, newHGap, _vgap);
						break;
			case 3: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
						int newVGap = ((Integer)value).intValue();
						if (newVGap <0) return XFDesignConstants.RSP_DIFF_VALUE;
						if (newVGap != _vgap)
							makeNewLayout(_rows, _cols, _hgap, newVGap);
						break;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
		return ret;
	}

	public void specialPropertyInput(Frame parent, XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyNoSpecialInputException,
		XFNameNotFoundException {
			if (index<0 || index >=propertyCount) throw new XFPropertyIndexOutOfBoundsException();
			else throw new XFPropertyNoSpecialInputException();
	}
		
	public String getClassName() {
		return "GridLayout";
	}

	///////////////////
	// Layout things //
	///////////////////

	public boolean allowsDragging() { return false; }

	public void layoutAdd(Container cont, XelfiVisualComponent comp) {
		cont.add((Component)comp);
	}

	public void layoutAdd(Container cont, int pos, XelfiVisualComponent comp) {
		cont.add((Component)comp, pos);
	}

	/////////////////////
	// Code generation //
	/////////////////////
	
	public String[] neededImports() {
		String list[] = new String[0];
		return list;
	}

	// this method returns whether it needs a variable for the layout
	public boolean genLayoutVariable() { return false; }
	
	public String genLayoutInitCode(String containerName, String varName) {
		if (containerName == null)
			if (_hgap == 0 && _vgap == 0)
				return "setLayout(new GridLayout(" + _rows+", "+_cols+"));\n";
			else
				return "setLayout(new GridLayout(" + _rows+", "+_cols+", " + _hgap+", "+_vgap+"));\n";
		else 
			if (_hgap == 0 && _vgap == 0)
				return containerName+".setLayout(new GridLayout(" + _rows+", "+_cols+"));\n";
			else
				return containerName+".setLayout(new GridLayout(" + _rows+", "+_cols+", " + _hgap+", "+_vgap+"));\n";
	}
	
	public String genComponentLayoutCode(String containerName, XelfiVisualComponent comp, String varName) {
		String compName = comp.getName();
		if (containerName == null)
			return "add("+compName+");";
		else 
			return containerName+".add("+compName+");";
	}
	/////////////////
	// Persistence //
	/////////////////

	public void loadFromStream(DataInput stream) throws
		IOException { 
		_rows = stream.readInt();
		_cols = stream.readInt();
		_hgap = stream.readInt();
		_vgap = stream.readInt();
		Debug.debug(Debug.FORMIO_LOAD, "Loaded GridBagLayout ["+_rows+", "+_cols+", "+_hgap+", "+_vgap+"]");
		makeNewLayout(_rows, _cols, _hgap, _vgap);
	}
	
	public void saveToStream(DataOutput stream) throws
		IOException { 
		stream.writeInt(_rows);
		stream.writeInt(_cols);
		stream.writeInt(_hgap);
		stream.writeInt(_vgap);
		Debug.debug(Debug.FORMIO_SAVE, "Saved GridBagLayout ["+_rows+", "+_cols+", "+_hgap+", "+_vgap+"]");
	}
}	

/* TODO
	1. properties 4&5 as Choice
*/