package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.util.Debug;

/**
 * Panel with absolute layout, layouts its children into one
 * column. Children will be wide as possible asn their height
 * is taken from their preffered sizes.
 * Children have to be subclasses of XFPropertyStringBox.
 *
 * @author Dafe
 * @version 1.00 Dec 21
	Last change:  DF   29 Jan 97    9:30 pm
*/
class XFNamesPanel extends Panel {
  // event id (selection changed)
  public static final int       SELECTION_CHANGED = 4855+1;
  public static final int       SELECTION_DOUBLE_CLICKED = 4855+10;
	private static final int 	    SPACE_BETWEEN = 0;

  protected Vector                propBoxes;
  protected XFPropertyStringBox   selected;
  protected Point                 startingPoint;

	private boolean									noReaction = true;

	XFNamesPanel () {
		setLayout(null);
		setBackground(Color.lightGray);
    propBoxes = new Vector();
    startingPoint = new Point(0, 0);
	}

  void setStartingPoint (int x, int y) {
    setStartingPoint(new Point(x, y));
  }

  void setStartingPoint (Point newPoint) {
    startingPoint = newPoint;
  }

	/**
	 * Sets starting point so that specified box is
	 * on the top of currently showed portion of
	 * panel.
	*/
	void bringBoxToTop (XFPropertyStringBox box) {
		int index = propBoxes.indexOf(box);
		if (index==-1) return;
		setStartingPoint(getStartingPoint().x, index);
	}

	/**
	 * Sets starting point so that specified box is
	 * on the down of currently showed portion of
	 * panel.
	*/
	void bringBoxToDown (XFPropertyStringBox box, int visibleHeight) {
		int index = propBoxes.indexOf(box);
		if (index==-1) return;
		setStartingPoint(getStartingPoint().x,
										 index-(visibleHeight/getBoxHeight()-1));
	}

  Point getStartingPoint () {
    return startingPoint;
  }

  void addProperty (XFPropertyStringBox psb) {
    propBoxes.addElement(psb);
  }

  void removeProperty (XFPropertyStringBox psb) {
    propBoxes.removeElement(psb);
  }

  void removeAllProps () {
    propBoxes.removeAllElements();
  }

  XFPropertyStringBox getProperty (int index) {
    for ( int i=0; i<propBoxes.size(); i++ ) {
      XFPropertyStringBox curProp = (XFPropertyStringBox)propBoxes.elementAt(i);
      if ( curProp.getIndex()==index ) return curProp;
    }
    return null;
  }

  void clearSelection () {
    selected = null;
  }

  void setSelection (XFPropertyStringBox selected) {
    this.selected = selected;
  }

  XFPropertyStringBox getSelection () {
    return selected;
  }

  /**
   * Returns next element to one currently selected.
   * Returns very first one if no selection occurs.
  */
  XFPropertyStringBox nextToSelected () {
    int propCount = propBoxes.size();
    if (propCount<=0) return null;
    int index;
    if (selected==null)
      index = 0;
    else {
      index = propBoxes.indexOf(selected);
      if (index<(propCount-1)) index++;
    }
    return (XFPropertyStringBox)propBoxes.elementAt(index);
  }

  /**
  * Returns previous element to one currently selected.
  * Returns very last one if no selection occurs.
  */
  XFPropertyStringBox previousToSelected () {
    int propCount = propBoxes.size();
    if (propCount<=0) return null;
    int index;
    if (selected==null)
      index = propCount-1;
    else {
      index = propBoxes.indexOf(selected);
      if (index>0) index--;
    }
    return (XFPropertyStringBox)propBoxes.elementAt(index);
  }

  /**
   * Returns upper left point of bounding rectangle 
   * of element with specified index
  */
  protected Point calculateUpperLeft (int index) {
    int totalHeight = getBoxHeight()*(-startingPoint.y);
    for (int i=0; i<index; i++) {
      XFPropertyStringBox curBox = 
          (XFPropertyStringBox)propBoxes.elementAt(i);
      totalHeight += curBox.preferredSize().height+SPACE_BETWEEN;
    }
    return new Point(-startingPoint.x, totalHeight);
  }

  /**
   * Returns upper left point of bounding rectangle
   * of specified element
  */
  protected Point calculateUpperLeft (XFPropertyStringBox box) {
    int totalHeight = getBoxHeight()*(-startingPoint.y);
    XFPropertyStringBox curBox =
        (XFPropertyStringBox)propBoxes.elementAt(0);
    for (int i=0; (i<propBoxes.size())&&
         (((XFPropertyStringBox)propBoxes.elementAt(i))!=box); i++) {
      curBox = (XFPropertyStringBox)propBoxes.elementAt(i);
      totalHeight += curBox.preferredSize().height+SPACE_BETWEEN;
    }
    return new Point(-startingPoint.x, totalHeight);
  }

  /**
   * Returns bounds rectangle of specified property box.
  */
  Rectangle calculateBounds (XFPropertyStringBox box) {
    Point upperLeft = calculateUpperLeft (box);
    Dimension size = box.preferredSize();
    return new Rectangle(upperLeft.x, upperLeft.y, size.width, size.height);
  }

	public void paint (Graphics g) {
		super.paint(g);
    Dimension mySize = size();
    // calls paint for each box in Vector
    int totalHeight = (-startingPoint.y)*getBoxHeight();
    for (int i=0; i<propBoxes.size(); i++) {
      XFPropertyStringBox curProp = (XFPropertyStringBox)propBoxes.elementAt(i);
      Dimension curPropSize = curProp.preferredSize();
      curProp.paint(g, -startingPoint.x, totalHeight, 
                    mySize.width, curPropSize.height);
      totalHeight += curPropSize.height+SPACE_BETWEEN;
    }
	}

  public boolean mouseDown (Event e, int x, int y) {
    XFPropertyStringBox found = findClickedPropertyBox(x, y);
    if ( found==null ) return false;
//		Debug.debug(Debug.INSPECTOR, found+"-"+e.clickCount);
		// catch double click on selected box
    if (found.equals(selected)) {
			if ((e.clickCount>2)||((!noReaction)&&(e.clickCount==2))) {
		    // modify event and let it propagate
	  	  e.id = SELECTION_DOUBLE_CLICKED;
  	 		e.arg = found;
    		return false;
			}
		}
    noReaction = !found.equals(selected);

/*      if (selected!=null) selected.select(false);
//      found.down();
//	    repaint();
    } */
//    selected = found.isSelected() ? found : null;
    // modify event and let it propagate
    e.id = SELECTION_CHANGED;
    e.arg = found;
    return false;
  }

  /**
   * returns preferred height for this panel.
  */
  int preferredHeight () {
    // sum preferred sizes of boxes
    int totalHeight = 0;
    for (int i=0; i<propBoxes.size(); i++) {
      XFPropertyStringBox curProp = (XFPropertyStringBox)propBoxes.elementAt(i);
      Dimension curPropSize = curProp.preferredSize();
      totalHeight += curPropSize.height+SPACE_BETWEEN;
    }
    return totalHeight;
  }

	/**
	 * Returns size of the box.
	 * Returns 0 if no box exists in panel.
	*/
	int getBoxHeight () {
    if (propBoxes.size()<=0)
			return 0;
		XFPropertyStringBox box = (XFPropertyStringBox)propBoxes.elementAt(0);
		return box.preferredSize().height;
	}

  protected XFPropertyStringBox findClickedPropertyBox (int x, int y) {
    int totalHeight = (-startingPoint.y)*getBoxHeight();
    Dimension mySize = size();
    for (int i=0; i<propBoxes.size(); i++) {
      XFPropertyStringBox curProp = (XFPropertyStringBox)propBoxes.elementAt(i);
      Dimension curPropSize = curProp.preferredSize();
      if ( new Rectangle(-startingPoint.x, totalHeight, 
                         mySize.width, curPropSize.height).inside(x, y))
        return curProp;
      totalHeight += curPropSize.height+SPACE_BETWEEN;
    }
    return null;
  }

}

