package xelfi.design.inspector;

import java.awt.*;

import xelfi.util.Debug;

/**
 * Plastic 3d decorational border. 
 * User can change width, colors, state of border.
 *
 * @author Dafe
 * @version 1.00, 21 Dec 1996
	Last change:  DF   21 Dec 96   11:18 pm
*/
class XFPlasticDecorator {
	static final public int PLASTIC_UP = 1;
 	static final public int PLASTIC_DOWN = 2;

	private int 				borderWidth;
	private int					type;
  private Color       light, shadow;

	XFPlasticDecorator () {
    this(PLASTIC_UP, 1, Color.white, Color.darkGray);
	}

	XFPlasticDecorator (int type, int borderWidth, Color light, Color shadow) {
		this.type = type;
		this.borderWidth = borderWidth;
		this.light = light;
		this.shadow = shadow;
	}

	void setWidth (int borderWidth) {
		this.borderWidth = borderWidth;
	}
	
	int getWidth () {
		return borderWidth;
	}

	void up () {
		setType(PLASTIC_UP);
	}

	void down () {
		setType(PLASTIC_DOWN);
	}

	void toggle () {
		setType(type==PLASTIC_UP ? PLASTIC_DOWN : PLASTIC_UP);
	}

	void setType (int type) {
		this.type = type;
	}

	int getType () {
		return type;
	}

  void setLightColor (Color light) {
    this.light = light;
  }

  void setShadowColor (Color shadow) {
    this.shadow = shadow;
  }

  Color getLightColor () {
    return light;
  }

  Color getShadowColor () {
    return shadow;
  }

  public void paint (Graphics g, Rectangle paintRect) {
    paint(g, paintRect.x, paintRect.y, paintRect.width, paintRect.height);
  }

  public void paint (Graphics g, int x, int y, int width, int height) {
    // choose right color and paint upper left portion
		g.setColor(type==PLASTIC_UP ? light : shadow);
		for (int i=0; i<borderWidth; i++) {
			g.drawLine(x+i, y+i, x+width-i-1, y+i);
			g.drawLine(x+i, y+i, x+i, y+height-i-1);
		}
    // paint lower right corner lines
		g.setColor(type==PLASTIC_UP ? shadow : light);
		for (int i=0; i<borderWidth; i++) {
			g.drawLine(x+i, y+height-i-1, x+width-i-1, y+height-i-1);
			g.drawLine(x+width-i-1, y+i, x+width-i-1, y+height-i-1);
		}
	}
}

