package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.design.components.*;
import xelfi.design.util.*;
import xelfi.top.*;
import xelfi.util.Debug;

/**
 * Class responsible for value boxes in CI. Shows different types of
 * awt controls when selected according to the viewer type of associated
 * component(s). Shows normal XFPropertyStringBox when it isn't selected.
 *
 * @author Dafe
 * @version 1.13, 2 Dec 1996
	Last change:  DF   29 Jan 97    7:09 pm
*/
class XFPropertyValueBox extends XFPropertyStringBox {

	// private attributes
	private XFCommonProperties		commonProperties;
	private int										viewerType;
  private Component							parent;

  private Color                 readOnlyColor = Color.gray,
                                multipleColor = Color.red;

	XFPropertyValueBox (Component parent, int index,
											XFCommonProperties commonProperties) {
    this(parent, index, Color.blue.darker(), null, commonProperties);
	}

	XFPropertyValueBox (Component parent, int index, Color color,
											XFCommonProperties commonProperties) {
    this(parent, index, color, null, commonProperties);
  }

	XFPropertyValueBox (Component parent, int index, Color color, Font font,
											XFCommonProperties commonProperties) {
    super(index, null, color, font);
    this.parent = parent;
		this.commonProperties = commonProperties;
		// get viewer type and set view mode by default
		viewerType = recognizeViewerType();
    refresh();
  }

	int getViewerType () {
		return viewerType;
	}

  XFCommonProperties getCommonProps () {
    return commonProperties;
  }

	/**
	 * selects or unselects box
	*/
	void select (boolean state) {
    boolean result = true;
    if ( state==true ) {
      if ( !isSelected() ) {
        result = changeToEditMode();
      }
    }
    else {
      if ( isSelected() )	changeToViewMode();
    }
    if ( result ) super.select(state);
	}


	public void paint (Graphics g, 
                     int x, int y, int width, int height) {
    if ( isSelected()==false )
  		super.paint(g, x, y, width, height);
	}

  void doSetProperty (Object newValue) {
    // set value into selected props
    int result = commonProperties.setProperties(getIndex(), newValue);
		// find out which properties need refreshing
		if (((result & XFDesignConstants.RSP_FORM_UPDATE)!=0) ||
				((result & XFDesignConstants.RSP_OTHER_PROP)!=0)) {
			// refresh other properties
			((XFValuesPanel)parent).refresh();
		}
	}

	void changeToViewMode () {
    // tell our parent that we are unselected
    ((XFValuesPanel)parent).changeToViewMode();
	}

	/**
	 * Performs an action of changing content into edit mode.
	 * Does not allow change to edit mode if property is marked as read only
	*/
	private boolean changeToEditMode () {
		// refuse editing when property is read-only
		if (viewerType==XFDesignConstants.PVT_READONLY) return false;
    // tell our parent that edit mode is on
    ((XFValuesPanel)parent).changeToEditMode(this, viewerType);
    return true;
  }

	String[] getPropertyValueNames () {
		String[] result = null;
		try {
			result = commonProperties.getPropertyValueNames(getIndex());
		} catch (XFPropertyIndexOutOfBoundsException e) {
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
		}
		return result;
	}

	/**
	 * Returns default event name for this property.
	 * Returns null if property is not an event property.
	*/
	String getDefaultEventName () {
		return commonProperties.getDefaultEventName(getIndex());
	}

	void refresh () {
		if (commonProperties==null) return;
		// sets string repr. of property value
		// or "<multiple>" word if values are not the same
		String result = commonProperties.getPropertiesString(getIndex());
    result = result==null ? "<multiple>" : result;
    setText(result);
    // set readonly color, if this is the case
    if (viewerType==XFDesignConstants.PVT_READONLY)
      setColor(readOnlyColor);
	}

	/**
	 * Recognizes a viewer type of the property currently being edited
	 * by this instance of value box and sets this type into
	 * "viewerType" instance variable
	*/
	private int recognizeViewerType () {
		int result = -1;
		try {
			result = commonProperties.getPropertyViewerType(getIndex());
		} catch (XFPropertyIndexOutOfBoundsException e) {
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
		} catch (XFNameNotFoundException e) {
			System.err.println(e.getMessage());
			e.printStackTrace(System.err);
		}
		return result;
	}


}
