package xelfi.design.inspector;

import java.awt.*;

import xelfi.util.Debug;

/**
 * Serves as a string box without any decorations. 
 * Text, color of text, text font can be changed.
 * Align is fixed to x-left, y-centered.
 * Background color can be set. (null==transparent)
 *
 * @author Dafe
 * @version 1.00, 21 Dec 1996
	Last change:  DF   24 Dec 96    3:19 pm
*/
class XFStringBox {

	// event constant
	public static final int		CLICKED = 4855+1;
	// drawing and sizing constants
	private static final int	LEFT_BORDER = 5;
	private static final int	TOP_BORDER = 2;
	private static final int	BOTTOM_BORDER = 1;
	private static final int	RIGHT_BORDER = 5;

	private String 						text = null;
	private Font							font = null;
	private Color							color;
	private FontMetrics				fm;
	private Dimension					prefSize;


	XFStringBox () {
		this(null, Color.black, null);
	}

	XFStringBox	(String text) {
		this(text, Color.black, null);
	}

	XFStringBox	(String text, Color color) {
		this(text, color, null);
	}

	XFStringBox	(String text, Color color, Font font) {
		super();
		this.text = text;
		this.color = color;
		this.font = font==null ? new Font("Helvetica", Font.PLAIN, 10) : font;
		fm = Toolkit.getDefaultToolkit().getFontMetrics(this.font);
		prefSize = new Dimension(0, 0);
		countPrefSize();
	}

	void setText (String text) {
		this.text = text;
	}

	String getText () {
		return text;
	}

	void setColor (Color color) {
		this.color = color;
	}
 
	Color getColor () {
		return color;
	}

  public Font getFont () {
    return font;
  }

	public void setFont (Font font) {
		this.font = font;
		fm = font==null ? null 
										: Toolkit.getDefaultToolkit().getFontMetrics(font);
		countPrefSize();
	}

	public Dimension preferredSize () {
		return prefSize;
	}	

	/**
	 * Paints text in specified color and font, 
	 * left aligned, vertically centered
	*/
	public void paint (Graphics g, int x, int y, int width, int height) {
		if (text==null) return;
    // set specified font, if possible
    Font curFont = g.getFont();
    if (!font.equals(curFont)) g.setFont(font);
    // set the color
    Color curColor = g.getColor();
    if (!color.equals(curColor)) g.setColor(color);
    // draw the string
    g.drawString(text, x+LEFT_BORDER,
                 y+((height-calculateHeight())/2+1)+fm.getAscent());
	}

	public void paint (Graphics g, Rectangle paintRect) {
    paint(g, paintRect.x, paintRect.y, 
          paintRect.width, paintRect.height);
  }

	/**
	 * Computes preferred size from font metrics and text string values.
	 * bug: prefSize variable must have been already created
	*/
	private void countPrefSize () {
		if (fm==null) return;
		prefSize.height = calculateHeight()+TOP_BORDER+BOTTOM_BORDER;
		prefSize.width = fm.stringWidth(text==null ? "" : text)+
                     LEFT_BORDER+RIGHT_BORDER;
	}

  /**
   * Calculates height.
   * Bug: fm must not be null
  */
  private int calculateHeight () {
    return fm.getAscent()+fm.getDescent();
  }

}
