package xelfi.design.inspector;

import java.awt.Frame;

import xelfi.design.components.xawt.XMenuBar;
import xelfi.design.forms.XelfiDesignInspectable;
import xelfi.awt.XelfiElementComparer;

/**
 * Main interface for XFMenuEditor class. XFMenuEditor class allows user 
 * to visually edit menus. XFMenuEditor shows menu drawn in "tree" form 
 * in its client area. XFMenuEditor has its own "command" menu for 
 * manipulating visually edited menu. It takes place in xelfi main menu,
 * submenu design.
 *
 * @author Dafe
 * version 1.3, 10 Dec 1996 - XelfiDesignInspectable added, 
 *                              setTarget, getTarget methods added
 *                              setMenu changed - second param vanished
 * version 1.31, 6 Feb 1997 - refresh method added
 *
 * @version 1.31, 6 Feb 1997
	Last change:  DF    6 Feb 97    0:55 am
*/
public interface XelfiMenuEditor extends XelfiDesignInspectable,
																					XelfiElementComparer {

	/**
	 * Asociates new XMenuBar object with MenuEditor window. Client area of
	 * Menu Editor is repainted. Second parameter "form" specifies form 
	 * object which will be notified when any part of menu changes in 
	 * Menu Editor.
	*/
	public void setMenu (XMenuBar menu);

	/**
	 * Sets new target. Target frame will receive menu as 
	 * user will visually construct it.
	*/
	public void setTarget (Frame target);

	/**
	 * Returns currently associated target.
	*/
	public Frame getTarget ();

	/**
	 * Returns currently asociated XMenuBar object.
	 * (XMenuBar is xelfi design component derived from java MenuBar)
	*/
	public XMenuBar getMenu ();

	/**
	 * Inserts new submenu to the menu.
	*/
	public void insertMenu ();

	/**
	 * Inserts new item to the menu.
	*/
	public void insertItem ();

	/**
	 * Inserts a separator to the menu. 
	*/	
	public void insertSeparator ();

	/**
	 * Removes item, submenu or separator from the menu.
	 * Operation should fail when we're trying to remove last item
	 * of submenu.
	*/
	public void remove ();

  /**
   * Removes menu from target.
   * This method is obviously called before change of properties
   * of menu components can occur.
  */
  public void removeMenu ();

  /**
   * Sets the menu to the target frame. 
   * Usually called after change of menu items' properties.
  */
  public void setMenu ();

	/**
	 * Refreshes content.
	*/
	public void refresh ();

}



