/**
* Xelfi
*/

package xelfi.design.util;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
* @version 	1.00, 24 Mar 1997
* @author 		Ian Formanek
*/

public class VersionInfo {
	private int major = 0;
	private int minor = 0;
	private int subminor = 0;

	public VersionInfo() {
	}
	
	public VersionInfo(int aMajor, int aMinor, int aSubminor) {
		major = aMajor;
		minor = aMinor;
		subminor = aSubminor;
	}
	
	public VersionInfo(int aMajor, int aMinor) {	
		major = aMajor;
		minor = aMinor;
	}

	public int getMajor() { return major; }
	public int getMinor() { return minor; }
	public int getSubminor() { return subminor; }
	
	public boolean equals(VersionInfo vi) {
		return (major == vi.getMajor() && minor == vi.getMinor() && subminor == vi.getSubminor());
	}
	
	public int compareTo(VersionInfo vi) {
		if (major < vi.getMajor()) return -1;
		else if (major > vi.getMajor()) return 1;
		else if (minor < vi.getMinor()) return -1;
		else if (minor > vi.getMinor()) return 1;
		else if (subminor < vi.getSubminor()) return -1;
		else if (subminor > vi.getSubminor()) return 1;
		else return 0;
	}
	
	public boolean isInside(VersionInfo v1, VersionInfo v2) {
		return (compareTo(v1)>=0 && compareTo(v2)<=0);
	}
	
	private String leadingZero(int num) {
		if (num <10) return "0"+num;
		else return ""+num;
	}

	private String leadingZeroes(int num) {
		if (num <10) return "00"+num;
		else if (num <100) return "0"+num;
		else return ""+num;
	}
	
	public String toString() {
		if (subminor == 0) return major+"."+leadingZero(minor);
		else return major+"."+leadingZero(minor)+"."+leadingZeroes(subminor);
	}
	
	public String toStringLong() {
		return major+"."+leadingZero(minor)+"."+leadingZeroes(subminor);
	}

	public void saveToStream(DataOutput stream) throws IOException {
		stream.writeInt(major);
		stream.writeInt(minor);
		stream.writeInt(subminor);
	}
	
	public void loadFromStream(DataInput stream) throws IOException {
		major = stream.readInt();
		minor = stream.readInt();
		subminor = stream.readInt();
	}
}