/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.*;
/**
* Outputs all characters that are refused by attached Acceptor to stream.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class DocumentInputStream extends InputStream {
  /** last position */
  Position last;

  /** current position */
  Position pos;

  /** upper or lower byte of char */
  boolean upper = true;

  /** Creates new stream.
  * @param f the first position
  * @param l the last position
  */
  DocumentInputStream (Position f, Position l) {
    pos = f;
    last = l;
  };

  /** Creates new stream. The end will be at the end of document.
  * @param f the first position
  */
  DocumentInputStream (Position f) {
    this (f, null);
  };
  
  /** Creates new stream.
  * @param b the block
  */
  DocumentInputStream (Block b) {
    this (b.getBegin (), b.getEnd ());
  };

  /** Reads one byte from stream.
  * @return the byte
  */
  public int read () {
    if (pos.equals (last)) {
      return -1;
    } else {
      try {
        return pos.getChar () & 0xff;
      } finally {
        try {
          pos = pos.nextChar ();
        } catch (EndOfTextException ex) {
          last = pos;
          // to signal end of text next time
        }
      }
    }
  }

  /** Returns the current position in document.
  * @return the current position
  */
  public final Position getPosition () {
    return pos;
  };
};

