/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.Dimension;
import java.awt.Color;
/**
* Interface that provides basic information about the way how to draw a
* document.
* Also called back when new line is inserted...
*
* @version 1.0
* @author Jaroslav Tulach
*/

public interface EditorCallback {
  /** Returns the height and width of every character used.
  * @return the height
  */
  public Dimension getCharDimension ();

  /** Returns the maximum ascent of any character used in document
  * @return the max ascent in points
  */
  public int getMaxAscent ();

  /** This method is called when a text has to be redrawn from
  * position p.
  * @param p the position of text that should be redrawn
  * @return normal TextDrawInfo extended with the information
  * in the field first that indicates the first position where
  * the text drawing should start (usually the same as p).
  */
  public StartDrawInfo beginDraw (Position p);


  /** This method is called when a new text has been
  * inserted between n and p.
  *
  * z@param n the begin of inserted text
  * @param p the end of inserted text
  * @return information about where to start redrawing (field
  * first - usually before n), where to stop redrawing (field
  * last - should be after p) and the information about how
  * to draw the first part of text
  * @warning the first and last positions should be on the same line
  * as positions n and/or p.
  * @see TextDrawInfo
  */
  public StartDrawInfo beginInsert (Position n, Position p);

  /** This method is called when a text s is deleted
  * from position p.
  *
  * @param p the position
  * @param s the deleted text
  * @return the information about where to start, where to
  * finish redrawing and which style to use at the
  * first part of text.
  * @warning the first and last positions should be on the same line
  * as position n.
  */
  public StartDrawInfo beginDelete (Position p, String s);

  /** Returns info about how the text from position p should be painted.
  * This method is called only when there is already a draw info
  * constructed by one of previous methods.
  *
  * @param di the previous draw info
  * @return the info about the text
  */
  public TextDrawInfo getDrawInfo (TextDrawInfo di);

  /** Returns default background that will be used in the editor's canvas.
  * Returning this value can avoid flickering in editor window.
  * @return the color or null
  */
  public Color getDefaultBackground ();
};




