/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;
/**
* Standard handler for documents.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class StandardDocumentHandler extends Object implements DocumentHandler {
  /** this instance is set to every editor */
  static StandardDocumentHandler standardDocumentHandler = new StandardDocumentHandler ();

  /** Saves the file by calling the save static method of this class.
  * @param s the source
  * @param backup true if a backup should be created
  * @exception IOException if some error occures
  */
  public void saveSource (Source s, boolean backup) throws IOException {
    save (s, backup);
  };
  /** Shows only the editor.
  */
  public void showAll (Source s) {
    // hack
    xelfi.top.TopLevel.showEditor (s, true);
  };


  /** This is default way how to save files.
  * @param s the source to save
  * @param os the stream to save the source to
  * @exception IOException if some error occures
  */
  public static void save (Source s, DataOutput os) throws IOException {
    File fileName = s.getSource ();
    s.getDocument ().save (os);
  };

  /** This is default way how to save files. The output file
  * stream is created for file Source.getSource () and the file
  * is saved to it.
  *
  * @param s the source to save
  * @param backup
  * @exception IOException if some error occures
  */
  public static void save (Source s, boolean backup) throws IOException {
    if (backup) {
      try {
        backup (s.getSource ());
      } catch (IOException ex) {
      };
    };
    DataOutputStream dos = new DataOutputStream (new FileOutputStream (s.getSource ()));
    save (s, dos);
    dos.close ();
  };

  /** Backups a file. Creates its copy with another extension.
  * @param f the file to backup.
  * @exception IOException if an error occures
  */
  public static void backup (File f) throws IOException {
    FileInputStream is = new FileInputStream (f);
    FileOutputStream os = new FileOutputStream (f.toString () + '~');

    byte[] b = new byte[1024];
    int len;
    while ((len = is.read (b, 0, 1024)) > 0) {
      os.write (b, 0, len);
    };
    is.close ();
    os.close ();
  };
};








