/**
* Xelfi Environment
*/

package xelfi.editor.keys;
import java.util.Vector;
import java.io.InputStream;

/** Operator with key redefinition tables.
*
* @author Jaroslav Tulach
* @version 1.0
*/
public abstract class KeyOperator extends Object {
  /** constants for modifiers */
  public static final int SHIFT = KeyCode.SHIFT;
  public static final int CTRL = KeyCode.CTRL;
  public static final int ALT = KeyCode.ALT;

  /** the basic ContextTable */
  static ContextTable basic;

  /** Loads new config to the system.
  * @param is stream with input file
  * @exception EditorKeyException
  */
  public static synchronized void load (InputStream is) throws EditorKeyException {
    ContextTable old = basic;
    basic = new ContextTable ();
    try {
      KeyConfig.parseConfig (is);
    } catch (ParseError ex) {
      // return back the previous state
      basic = old;
      throw new EditorKeyException (ex.getMessage ());
    };
  };

  /** Gets new table for handling keys.
  * @return new table
  */
  public static synchronized KeyTable newTable () {
    return new KeyTable ();
  };

  /** Adds new handler for the key.
  * @param key the key
  * @param v KeyHandlerMacro
  * @return false if something is wrong
  */
  static boolean addKey (KeyCode key, KeyActionMacro v) {
    return basic.addKey (key, v);
  };

  /** Converts whole table to string.
  * @return string that describes data
  */
  public static String tableToString () {
    return basic.toString ();
  };
}





// Generated from substitution template 'XelfiClass' Blocks:Class:22,13,33,13; Package:8,4,25,4;





