/**
* Xelfi Environment
*/

package xelfi.editor.keys;
import java.io.InputStream;
import java.util.NoSuchElementException;
import xelfi.editor.*;

/** Table with keys. Can be loaded from configuration file.
* Distributes keys to handlers.

* @see KeyActionHandler
*
* @author Jaroslav Tulach
* @version 1.0
*/
public class KeyTable extends Object {
  /** is the table that is active in this key context */
  ContextTable current;

  /** Initializes the table key
  */
  KeyTable () {
    init ();
  };

  /** Initializes the table key
  */
  public void init () {
    current = KeyOperator.basic;
  };

  /** Distributes this key to the right handler
  * @param key the key code
  * @param modifiers state of SHIFT,
  * @return true if default action (insert) has been provided
  * @exception NotAllowedException if the insert is not allowed
  * @exception EndOfTextException if end of text is reached
  */
  public boolean keyDown (int key, int modifiers, TextEditorInfo info)
  throws NotAllowedException, EndOfTextException {
    // use default set only on the first level
    boolean defSet = current == KeyOperator.basic;
    KeyCode k = new KeyCode (modifiers, 0/*ignore*/, key);
    ContextTable newValue = KeyOperator.basic;
    try {
      newValue = current.distributeKey (k, info);
      // key handled no default set
      defSet = false;
    } catch (NoSuchElementException ex) {
      // the key cannot be handled, report true
    } finally {
      current = newValue;
    };
    return defSet;
  };
}




// Generated from substitution template 'Class' Blocks:Class:13,17,21,17; Package:8,4,25,4; 
