/** Column layout manager
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;
import java.util.*;

  /** This layout layout components. Each component occupies one column. */
public class ColLayout extends LayoutBase
{
  public void layoutContainer(Container parent)
  {
    int container = parent.size().width, last = components.size() - 1, i;
    Insets conti = parent.insets();
    Dimension prefsize[] = prefSizes();
      // try ideal size
    int cols[] = preferredWidth(prefsize), left[] = new int[last + 1], right[] = new int[last + 1];
    int dif = container - conti.left - conti.right - cols[last + 1] ;  // total width
    if(dif > 0) distributeSpace(dif, cols, left, right);  // more space than we need
    else
    {  // too little space
      cols = minimumWidth();
      dif = container - conti.left - conti.right - cols[last + 1];  // total width
      if(dif > 0) distributeSpace(dif, cols, left, right);   // it helped
      else for(i = 0; i <+ last; i++) left[i] = right[i] = 0;  // little space again
    }

    int x = conti.left, y = conti.top, contheight = parent.size().height - conti.bottom - conti.top, compheight;
    Insets compi = null;
    LayoutPropertiesBase pr = null;
    for(i = 0; i <= last; i++)
    {
        // left space
      x += left[i];
        // component
      pr = (LayoutPropertiesBase)prop.elementAt(i);
      compi= pr.compInsets;
      y = conti.top + compi.top;
      compheight = contheight - compi.top - compi.bottom;
//       System.out.println("component dif: " + (compheight - prefsize[i].height));
      if((dif = compheight - prefsize[i].height) > 0)
      {                                      // distribute space horizontally
        int l = (int)(pr.topSpaceGrowth * dif);
        y += l;
        compheight -= l;
        compheight -= (int)(pr.bottomSpaceGrowth * dif);
      }
      ((Component)components.elementAt(i)).reshape(x + compi.left, y, cols[i] - compi.left - compi.right, compheight);
//      System.out.println((x + compi.left) + "   " + y  + "   " +( cols[i] - compi.left - compi.right)  + "   " + compheight));

        // component + bottom space
      x += cols[i] + right[i];
    }
  }

  public Dimension minimumLayoutSize(Container  parent)
  {
    int height = 0;
    Dimension minsize[] = minSizes();
    int last = components.size(), i;
    Insets compi = null;
      // count max height
    for(i = 0; i < last; i++)
    {
      compi = ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      height = Math.max(minsize[i].height +  compi.top + compi.bottom, height);
    }
      // count width
    int cols[] = minimumWidth(minsize);
    Insets ins = parent.insets();
    return new Dimension(cols[last] + ins.left + ins.right, height + ins.top + ins.bottom);
  }

  public Dimension preferredLayoutSize(Container  parent)
  {
    int height = 0;
    Dimension prefsize[] = prefSizes();
    int last = components.size(), i;
    Insets compi = null;
      // count max height
    for(i = 0; i < last; i++)
    {
      compi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      height = Math.max(prefsize[i].height + compi.top + compi.bottom, height);
    }
      // count width
    int cols[] = preferredWidth(prefsize);
    Insets ins = parent.insets();
    Dimension d = new Dimension(cols[last] + ins.left + ins.right, height + ins.top + ins.bottom);
//    System.out.println("Container insets: " + (ins.top + ins.bottm));
//    System.out.println("Container prefheight: " + d.height);
    return d;

  }

    /** Builds an array of columns. Last field is total width. Array contains widths of rows.
      Input parameter is an array of preferred sizes of  components.
    */
  protected int[] preferredWidth(Dimension prefsize[])
  {
    Component pom = null;
    Insets pomi = null;
    int l = components.size(), i, total = 0;
    int cols[] = new int[l + 1];
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pomi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      total += (cols[i] = prefsize[i].width + pomi.left + pomi.right);
    }
    cols[l] = total;
    return cols;
  }

    /** Builds an array of cols. Last field is total width. Array contains height of cols.
      Input parameter is an array of minimum sizes of  components.
    */
  protected int[] minimumWidth(Dimension minsize[])
  {
    Component pom = null;
    Insets pomi = null;
    int l = components.size(), i, total = 0;
    int cols[] = new int[l + 1];
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pomi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      total += (cols[i] = minsize[i].width + pomi.left + pomi.right);
    }
    cols[l] = total;
    return cols;
  }

    /** Builds an array of cols. Last field is total width. Array contains height of cols. */
  protected int[] minimumWidth()
  {
    return minimumWidth(minSizes());
  }

    /** Distributes space also into left and rihgt column space. */
  protected void distributeSpace(int space, int cols[], int leftSpace[], int rightSpace[])
  {
    Component pom = null;
    LayoutPropertiesBase pr = null;
    int l = components.size(), i;
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pr = (LayoutPropertiesBase)prop.elementAt(i);
      cols[i] += space * pr.compGrowth;
      leftSpace[i] = (int)(space * pr.leftSpaceGrowth);
      rightSpace[i] = (int)(space * pr.rightSpaceGrowth);
    }
  }
}

